package flm.b4a.betterimageview;

import java.io.IOException;
import java.io.InputStream;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.LightingColorFilter;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.BitmapDrawable;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.drawable.CanvasWrapper.BitmapWrapper;
import anywheresoftware.b4a.objects.streams.File;

@BA.ActivityObject
@BA.Version(1.1F)
@BA.Author("Frdric Leneuf-Magaud")
@BA.ShortName("BetterImageView")
@BA.Events(values = {
		"Touch(Action As Int, X As Int, Y As Int)",
		"Click",
		"LongClick"
})
public class BetterImageViewWrapper extends ViewWrapper<BetterImageView> implements DesignerCustomView
{
	@BA.Hide
	public void innerInitialize(final BA ba, final String eventName, boolean keepOldObject)
	{
		if (!keepOldObject) {
			setObject(new BetterImageView(ba.context));
		}
		super.innerInitialize(ba, eventName, true);
		if (ba.subExists(eventName + "_touch")) {
			getObject().setOnTouchListener(new ImageView.OnTouchListener() {
				@Override
				public boolean onTouch(View v, MotionEvent ev) {
					if (getEnabled())
						ba.raiseEvent(getObject(), eventName + "_touch",
								new Object[] { ev.getAction() & MotionEvent.ACTION_MASK, (int)ev.getX(), (int)ev.getY() });
					return true;
				}
			});
		}
	}

	/**
	 *Initializes the BetterImageView.
	 *<B>EventPrefix</B> prefix of events.
	 */
	public void Initialize(BA ba, String EventPrefix)
	{
		innerInitialize(ba, EventPrefix.toLowerCase(BA.cul), false);
	}

	public static final ImageView.ScaleType SCALETYPE_CENTER = ImageView.ScaleType.CENTER;
	public static final ImageView.ScaleType SCALETYPE_CENTER_CROP = ImageView.ScaleType.CENTER_CROP;
	public static final ImageView.ScaleType SCALETYPE_CENTER_INSIDE = ImageView.ScaleType.CENTER_INSIDE;
	public static final ImageView.ScaleType SCALETYPE_FIT_CENTER = ImageView.ScaleType.FIT_CENTER;
	public static final ImageView.ScaleType SCALETYPE_FIT_END = ImageView.ScaleType.FIT_END;
	public static final ImageView.ScaleType SCALETYPE_FIT_START = ImageView.ScaleType.FIT_START;
	public static final ImageView.ScaleType SCALETYPE_FIT_XY = ImageView.ScaleType.FIT_XY;

	/**
	 *Gets or sets how the foreground image should be resized or moved to match the size of this BetterImageView.
	 *The value is one of the SCALETYPE constants.
	 */
	public ImageView.ScaleType getScaleType()
	{
		return getObject().getScaleType();
	}
	public void setScaleType(ImageView.ScaleType NewScaleType)
	{
		getObject().setScaleType(NewScaleType);
	}

	/**
	 *Gets or sets the gravity of the background image.
	 *Only bitmaps are affected by gravity.
	 */
	public int getGravity()
	{
		return getObject().Gravit;
	}
	public void setGravity(int value)
	{
		getObject().Gravit = value;
		Drawable d = getObject().getBackground();
		if (d != null && d instanceof BitmapDrawable) {
			((BitmapDrawable)d).setGravity(value);
		}
	}

	/**
	 *Changes the alpha value of the foreground image.
	 *<B>Alpha</B> an integer between 0 (fully transparent) and 255 (fully opaque).
	 */
	public void ChangeAlpha(int Alpha)
	{
		Drawable d = getObject().getDrawable();
		if (d != null) {
			d.setAlpha(Alpha);
		}
	}

	/**
	 *Changes the alpha value of the background image.
	 *<B>Alpha</B> an integer between 0 (fully transparent) and 255 (fully opaque).
	 */
	public void ChangeBackgroundAlpha(int Alpha)
	{
		Drawable d = getObject().getBackground();
		if (d != null) {
			d.setAlpha(Alpha);
		}
	}

	/**
	 *Gets or sets the foreground bitmap.
	 *Returns Null if the image is not a bitmap.
	 */
	public Bitmap getBitmap()
	{
		Drawable d = getObject().getDrawable();
		if (d == null || !(d instanceof BitmapDrawable)) {
			return null;
		}
		return ((BitmapDrawable)d).getBitmap();
	}
	public void setBitmap(Bitmap value)
	{
		getObject().setImageBitmap(value);
	}

	/**
	 *Gets or sets the foreground drawable.
	 *Returns Null if there is no image.
	 */
	public Drawable getDrawable()
	{
		return getObject().getDrawable();
	}
	public void setDrawable(Drawable d)
	{
		getObject().setImageDrawable(d);
	}

	/**
	 *Gets or sets the background drawable.
	 *Returns Null if there is no image.
	 */
	public Drawable getBackground()
	{
		return getObject().getBackground();
	}
	public void setBackground(Drawable d)
	{
		if (d == null) {
			getObject().setBackgroundDrawable(null);
			return;
		}
		if (d instanceof BitmapDrawable)
			((BitmapDrawable) d).setGravity(getObject().Gravit);
		getObject().setBackgroundDrawable(d);
	}

	/**
	 *Gets or sets the background bitmap.
	 *Returns Null if the image is not a bitmap.
	 */
	public Bitmap getBackgroundBitmap()
	{
		Drawable d = getObject().getBackground();
		if (d != null && d instanceof BitmapDrawable) {
			return ((BitmapDrawable)d).getBitmap();
		}
		return null;
	}
	public void setBackgroundBitmap(Bitmap value)
	{
		if (value == null) {
			getObject().setBackgroundDrawable(null);
			return;
		}
		BitmapDrawable bd = new BitmapDrawable(BA.applicationContext.getResources(), value);
		bd.setGravity(getObject().Gravit);
		getObject().setBackgroundDrawable(bd);
	}

	/**
	 *Changes the hue and the saturation of the given bitmap.
	 *@param <B>Hue</B> from 0 to 360. 0 or 360 = no change.
	 *@param <B>Saturation</B> 0 = gray-scale, 1 = no change.
	 */
	public static Bitmap AlterColors(BitmapWrapper Source, int Hue, float Saturation)
	{
		if ((Hue == 0 || Hue == 360) && Saturation == 1) {
			return Source.getObject();
		}
		ColorMatrix cm = new ColorMatrix();
		if (Saturation != 1)
			cm.setSaturation(Saturation);
		if (Hue != 0) {
			final float NewHue = (float) Hue / 180f * (float) Math.PI;
			final float cosVal = (float) Math.cos(NewHue);
			final float sinVal = (float) Math.sin(NewHue);
			final float lumR = 0.213f;
			final float lumG = 0.715f;
			final float lumB = 0.072f;
			float[] mat = new float[] { 
					lumR + cosVal * (1 - lumR) + sinVal * (-lumR), lumG + cosVal * (-lumG) + sinVal * (-lumG), lumB + cosVal * (-lumB) + sinVal * (1 - lumB), 0, 0, 
					lumR + cosVal * (-lumR) + sinVal * (0.143f), lumG + cosVal * (1 - lumG) + sinVal * (0.140f), lumB + cosVal * (-lumB) + sinVal * (-0.283f), 0, 0,
					lumR + cosVal * (-lumR) + sinVal * (-(1 - lumR)), lumG + cosVal * (-lumG) + sinVal * (lumG), lumB + cosVal * (1 - lumB) + sinVal * (lumB), 0, 0, 
					0f, 0f, 0f, 1f, 0f, 
					0f, 0f, 0f, 0f, 1f };
			cm.postConcat(new ColorMatrix(mat));
		}
		Bitmap bmp = Bitmap.createBitmap(Source.getObject());
		Canvas canvas = new Canvas(bmp);
		Paint p = new Paint();
		ColorMatrixColorFilter cmFilter = new ColorMatrixColorFilter(cm);
		p.setColorFilter(cmFilter);
		canvas.drawBitmap(bmp, 0.0F, 0.0F, p);
		return bmp;
	}

	/**
	 * Applies a color filter to the specified bitmap that multiplies the RGB channels by one color, and then adds a second color, pinning the result for each component to [0..255].
	 */
	public static Bitmap ApplyColorFilter(BitmapWrapper Source, int Multiply, int Add) throws Exception
	{
		Bitmap bmp = Bitmap.createBitmap(Source.getObject());
		Canvas canvas = new Canvas(bmp);
		Paint p = new Paint();
		p.setColorFilter(new LightingColorFilter(Multiply, Add));
		canvas.drawBitmap(bmp, 0.0F, 0.0F, p);
		return bmp;
	}

	/**
	 *Loads and rescales a bitmap.
	 *Throws an exception if there's not enough memory to load the specified file.
	 *@param <B>NewWidth</B> the new bitmap's desired width or -1 to keep the original width.
	 *@param <B>NewHeight</B> the new bitmap's desired height or -1 to keep the original height.
	 *@param <B>Filter</B> if True, improves the sampling quality.
	 */
	public static Bitmap LoadScaledBitmap(String Dir, String FileName, int NewWidth, int NewHeight, boolean Filter) throws IOException
	{
		// Lecture des dimensions de l'image
		File.InputStreamWrapper in = File.OpenInput(Dir, FileName);
		BitmapFactory.Options opt = new BitmapFactory.Options();
		opt.inJustDecodeBounds = true;
		BitmapFactory.decodeStream((InputStream)in.getObject(), null, opt);
		in.Close();

		// Calcul du ratio d'affichage
		int ratio = opt.outWidth / NewWidth;
		ratio = Math.max(ratio, opt.outHeight / NewHeight);
		BitmapFactory.Options opt2 = null;
		if (ratio > 1) {
			opt2 = new BitmapFactory.Options();
			opt2.inSampleSize = ratio;
		}

		// Dcodage du fichier
		in = File.OpenInput(Dir, FileName);
		Bitmap bmp;
		try {
			bmp = BitmapFactory.decodeStream((InputStream)in.getObject(), null, opt2);
		}
		catch (OutOfMemoryError OoMe) {
			throw new RuntimeException("LoadScaledBitmap: Out of memory.");
		}
		in.Close();
		if (bmp == null)
			throw new RuntimeException("LoadScaledBitmap: Error loading bitmap.");

		// Redimensionnement de l'image
		if (NewWidth == -1 && NewHeight == -1)
			return bmp;
		else {
			if (NewWidth == -1)
				NewWidth = bmp.getWidth();
			if (NewHeight == -1)
				NewHeight = bmp.getHeight();
			return Bitmap.createScaledBitmap(bmp, NewWidth, NewHeight, Filter);
		}
	}

	/**
	 * Gets a 9-patch drawable from the application resources.
	 */
	public static Drawable LoadNinePatchDrawable(BA ba, String Name)
	{
		int ID_drawable = ba.context.getResources().getIdentifier(Name, "drawable", ba.context.getPackageName());
		return ba.context.getResources().getDrawable(ID_drawable);
	}

	/**
	 *Converts a bitmap from the format ARGB_8888 to the format RGB_565 to save memory.
	 *This function removes the alpha channel and therefore the transparent areas become opaque.
	 */
	public static Bitmap ReduceColors(BitmapWrapper srcBmp)
	{
		return srcBmp.getObject().copy(Bitmap.Config.RGB_565, true);
	}

	// ----------------------------- B4A CUSTOM VIEW WITH DESIGNER SUPPORT -----------------------------
	@BA.Hide
	@Override
	public void _initialize(BA ba, Object activityClass, String EventName)
	{
		Initialize(ba, EventName);
	}

	@Override
	public void DesignerCreateView(PanelWrapper base, LabelWrapper lw, anywheresoftware.b4a.objects.collections.Map props)
	{
		// Remplacement du Panel par la BetterImageView
		ViewGroup vg = (ViewGroup) base.getObject().getParent();
		vg.addView(getObject(), new BALayout.LayoutParams(base.getLeft(), base.getTop(), base.getWidth(), base.getHeight()));

		// Mise  jour des proprits
		setGravity(lw.getGravity());
		setBackground(base.getBackground());
		setEnabled(lw.getEnabled());
		setVisible(lw.getVisible());
		setTag(lw.getTag());

		// Suppression du Panel
		base.RemoveView();
		base = null;
	}
}
